
set tclvars {}

set allFlags ""
set allVars ""

# The following three procs, together w/ the initialization 
# for the flags below, allow a totally Tcl-controlled menu to 
# shadow Alpha's internal variables.
proc initFlag {var {val ""}} {
	global $var modeVars tclvars allFlags
	if {[lsearch -exact $tclvars $var] < 0} {
		lappend tclvars $var
	}
	linkVar $var
	if {[string length $val]} {set $var $val}
	lappend allFlags $var
}

# Called for ordinary tcl global vars we want included in the Flags menu.
proc initTclFlag {flag {val 0}} {
	global $flag allFlags tclvars modeVars
	if {[lsearch -exact $tclvars $flag] < 0} {
		lappend tclvars $flag
	}
	if {![info exists $flag]} {set $flag $val}
	lappend allFlags $flag
}

# Called for ordinary tcl global vars we want included in the Vars menu.
proc initTclVar {var {val ""}} {
	global $var tclvars modeVars allVars
	if {[lsearch -exact $tclvars $var] < 0} {
		lappend tclvars $var
	}
	if {[string length $val]} {
		set $var $val
	}
	if {![info exists $var]} {set $var 0}
	lappend allVars $var
}

proc initVar {var {val ""}} {
	global $var tclvars modeVars allVars
	if {[lsearch -exact $tclvars $var] < 0} {
		lappend tclvars $var
	}
	linkVar $var
	if {[string length $val]} {
		set $var $val
	}
	lappend allVars $var
}

# 'fullNames' needs to be traced to update win menu.
if {![info exists oldFNames]} {set oldFNames 0}
proc shadowFullNames args {
	global winNameToNum fullNames oldFNames winMenu
	
	if {$fullNames == $oldFNames} {return $fullNames}
	set oldFNames $fullNames
	
	if {[catch {set nms [array names winNameToNum]}] == "1"} {
		return $fullNames
	}
	
	if {$fullNames == "1"} {
		foreach name $nms {
			if {$winNameToNum($name) < 10} {
				set lab /$winNameToNum($name)
			} else {set lab ""}
			regexp {[^:]*$} $name nm
			catch {deleteMenuItem -m $winMenu $nm}
			addMenuItem -m -l $lab $winMenu $name
		}
	} else {
		foreach name $nms {
			if {$winNameToNum($name) < 10} {
				set lab /$winNameToNum($name)
			} else {set lab ""}
			regexp {[^:]*$} $name nm
			catch {deleteMenuItem -m $winMenu $name}
			addMenuItem -m -l $lab $winMenu $nm
		}
	}
	return $fullNames
}

# This file contains methods allowing the shadowing of tcl variables 

initFlag autoHScroll
initFlag autoSave
initTclFlag backup
initFlag blinkingCursor
initFlag blockCursor
initFlag bwWindows
initFlag coloring
initFlag dragAndDrop
initTclFlag elecLBrace
initTclFlag elecRBrace
initTclFlag electricSemi
initFlag fullNames
trace vdelete fullNames w shadowFullNames
trace variable fullNames w shadowFullNames
initFlag horScrollBar
initFlag hyperText
initTclFlag iconifyOnSwitch
initFlag indentOnCR
initTclFlag infoWindowsDirty 1
initTclFlag intelCutPaste 1
catch {initFlag lockStatus}
initFlag maskCapsLock 1
initFlag moveInsertion
initFlag noDelete
initFlag openAllFiles
initFlag optionIsMeta
initFlag posActivate
initFlag powerThumb
initFlag projectorAware
initFlag showRowCol
initFlag showInvisibles
initFlag showMode
catch {initFlag showPlatform}
initTclFlag smartQuotes 1
initFlag sortedIsDefault
initFlag suppressHeader
initFlag tempMem
initFlag undoOn
initFlag wordWrap

initTclVar backExtension "~"
initTclVar backDir
set backDir ""
initVar bottomMargin
initVar changesLim
initVar defaultFont
initVar defHeight
initVar defLeft 	3
initVar defTop 		40
initVar defWidth
initTclVar diffFlags { }
initVar fillColumn
initVar fontSaving
initVar fontSize
initVar funcExpr
initVar funcPar
initVar leftFillColumn
initVar leftMargin
initTclVar numWinsToTile 2
initVar paraColumn
initVar	printerFont
initVar	printerFontSize
initVar savedState
initVar sortColumn
initVar tabSize
initVar tagFile
initTclVar tileHeight
initTclVar tileLeft 4
initTclVar tileTop
initTclVar tileWidth
initVar topMargin
initVar wordBreak
initVar wordBreakPreface

# Redefine 'menuHook' to update flags menu.
proc menuHook {} {
	uplevel #0 {
		foreach f $allFlags {
			markMenuItem globalFlags $f [set $f]
		}
	}
}
