#############################################################################
#############################################################################
#
# Special LaTeX Key Bindings (called from latex.tcl)
#
# abbreviations:  <o> = option, <z> = control, <s> = shift, <c> = command
#
#############################################################################
#
# Author:  Tom Scavo (trscavo@syr.edu)
#
#############################################################################
#############################################################################

# Initial binding of LaTeX command keys.
proc bindLaTeXKeys {} {
	global TeXmodeVars
	if {$TeXmodeVars(optionIsMeta)} then {
		toggleTeXKeys "bind" "z"
	} else {
		toggleTeXKeys "bind" ""
	}
	# Navigation:
	bind Kpad1 prevSubsection "TeX"
	bind Kpad3 nextSubsection "TeX"
	bind Kpad1 <s> prevSection "TeX"
	bind Kpad3 <s> nextSection "TeX"
	# tab:
	bind	'\t'	nextTabStop	"TeX"
	bind	'\t'	<s>	prevTabStop	"TeX"
	bind	'\t'	<z>	{nthTabStop 0}	"TeX"
	bind	'\t'	<c>	clearTabStops	"TeX"
}

# The proc 'shadowUseDollarSigns' is executed automatically 
# whenever the flag 'useDollarSigns' is altered.
trace variable TeXmodeVars(useDollarSigns) w shadowUseDollarSigns
proc shadowUseDollarSigns {name1 name2 op} {
	global TeXmodeVars
	if {$TeXmodeVars(optionIsMeta)} then {
		if {$TeXmodeVars(useDollarSigns)} then {
			unbind '4' <coz>	latexMath	"TeX"
			unbind '4' <scoz>	latexDisplaymath	"TeX"
			bind '4' <coz>	texMath	"TeX"
			bind '4' <scoz>	texDisplaymath	"TeX"
		} else {
			unbind '4' <coz>	texMath	"TeX"
			unbind '4' <scoz>	texDisplaymath	"TeX"
			bind '4' <coz>	latexMath	"TeX"
			bind '4' <scoz>	latexDisplaymath	"TeX"
		}
	} else {
		if {$TeXmodeVars(useDollarSigns)} then {
			unbind '4' <co>	latexMath	"TeX"
			unbind '4' <sco>	latexDisplaymath	"TeX"
			bind '4' <co>	texMath	"TeX"
			bind '4' <sco>	texDisplaymath	"TeX"
		} else {
			unbind '4' <co>	texMath	"TeX"
			unbind '4' <sco>	texDisplaymath	"TeX"
			bind '4' <co>	latexMath	"TeX"
			bind '4' <sco>	latexDisplaymath	"TeX"
		}
	}
}

# The proc 'shadowOptionIsMeta' is executed automatically 
# whenever the flag 'optionIsMeta' is altered.
trace variable TeXmodeVars(optionIsMeta) w shadowOptionIsMeta
proc shadowOptionIsMeta {name1 name2 op} {
	global TeXmodeVars
	if {$TeXmodeVars(optionIsMeta)} then {
		toggleTeXKeys "unbind" ""
		toggleTeXKeys "bind" "z"
	} else {
		toggleTeXKeys "unbind" "z"
		toggleTeXKeys "bind" ""
	}
}

# Bind or unbind the TeX keys, depending on the value of func.  
# The value of mod is a valid modifier key, either "z" or "" 
# in this case.  Note:  Any bindings containing a "z" designator 
# are listed first and therefore NOT available when 'optionIsMeta' 
# is set to true.
proc toggleTeXKeys {func mod} {
	global TeXmodeVars
	
	eval $func	'a'	<oz$mod>	forall	"TeX"
	eval $func	'a'	<o$mod>	alpha	"TeX"
	eval $func	'a'	<os$mod>	angle	"TeX"
	eval $func	'a'	<co$mod>	acute	"TeX"
	
	eval $func	'b'	<oz$mod>	Box	"TeX"
	eval $func	'b'	<o$mod>	beta	"TeX"
	eval $func	'b'	<co$mod>	bar	"TeX"
	eval $func	'b'	<cs$mod>	textbf	"TeX"
	eval $func	'b'	<cso$mod>	mathbf	"TeX"
	
	eval $func	'c'	<o$mod>	chi	"TeX"
	eval $func	'c'	<co$mod>	check	"TeX"
	eval $func	'c'	<cs$mod>	cite	"TeX"
	eval $func	'c'	<cso$mod>	mathcal	"TeX"
	
	eval $func	'd'	<oz$mod>	Diamond	"TeX"
	eval $func	'd'	<o$mod>	delta	"TeX"
	eval $func	'd'	<os$mod>	Delta	"TeX"
	eval $func	'd'	<co$mod>	dot	"TeX"
	eval $func	'd'	<cso$mod>	ddot	"TeX"
	
	eval $func	'e'	<oz$mod>	varepsilon	"TeX"
	eval $func	'e'	<o$mod>	epsilon	"TeX"
	eval $func	'e'	<os$mod>	exists	"TeX"
	eval $func	'e'	<cs$mod>	emph	"TeX"
	
	eval $func	'f'	<oz$mod>	varphi	"TeX"
	eval $func	'f'	<o$mod>	phi	"TeX"
	eval $func	'f'	<os$mod>	Phi	"TeX"
	eval $func	'f'	<co$mod>	fraction	"TeX"
	eval $func	'f'	<cs$mod>	footnote	"TeX"
	
	eval $func	'g'	<oz$mod>	copyright	"TeX"
	eval $func	'g'	<o$mod>	gamma	"TeX"
	eval $func	'g'	<os$mod>	Gamma	"TeX"
	eval $func	'g'	<co$mod>	grave	"TeX"
	
	eval $func	'h'	<o$mod>	eta	"TeX"
	eval $func	'h'	<co$mod>	hat	"TeX"
	eval $func	'h'	<cs$mod>	textsc	"TeX"
	eval $func	'h'	<cso$mod>	widehat	"TeX"
	
	eval $func	'i'	<oz$mod>	imath	"TeX"
	eval $func	'i'	<o$mod>	iota	"TeX"
	eval $func	'i'	<co$mod>	int	"TeX"
	eval $func	'i'	<cs$mod>	textit	"TeX"
	eval $func	'i'	<cso$mod>	mathit	"TeX"
	
	eval $func	'j'	<oz$mod>	jmath	"TeX"
	eval $func	'j'	<o$mod>	partial	"TeX"
	
	eval $func	'k'	<o$mod>	kappa	"TeX"
	
	eval $func	'l'	<oz$mod>	ell	"TeX"
	eval $func	'l'	<o$mod>	lambda	"TeX"
	eval $func	'l'	<os$mod>	Lambda	"TeX"
	eval $func	'l'	<cs$mod>	label	"TeX"
	
	eval $func	'm'	<oz$mod>	mapsto	"TeX"
	eval $func	'm'	<o$mod>	mu	"TeX"
	eval $func	'm'	<co$mod>	mbox	"TeX"
	eval $func	'm'	<cs$mod>	mbox	"TeX"
	
	eval $func	'n'	<oz$mod>	cap	"TeX"
	eval $func	'n'	<o$mod>	nu	"TeX"
	eval $func	'n'	<os$mod>	aleph	"TeX"
	eval $func	'n'	<co$mod>	nthRoot	"TeX"
	eval $func	'n'	<cs$mod>	marginalNote	"TeX"
	
	eval $func	'o'	<oz$mod>	bigcirc	"TeX"
	eval $func	'o'	<o$mod>	omicron	"TeX"
	eval $func	'o'	<os$mod>	circ	"TeX"
	eval $func	'o'	<co$mod>	overline	"TeX"
	eval $func	'o'	<cso$mod>	overbrace	"TeX"
	
	eval $func	'p'	<oz$mod>	varpi	"TeX"
	eval $func	'p'	<o$mod>	pi	"TeX"
	eval $func	'p'	<os$mod>	Pi	"TeX"
	eval $func	'p'	<co$mod>	prod	"TeX"
	eval $func	'p'	<cs$mod>	pageref	"TeX"
	
	eval $func	'q'	<oz$mod>	vartheta	"TeX"
	eval $func	'q'	<o$mod>	theta	"TeX"
	eval $func	'q'	<os$mod>	Theta	"TeX"
	
	eval $func	'r'	<oz$mod>	varrho	"TeX"
	eval $func	'r'	<o$mod>	rho	"TeX"
	eval $func	'r'	<co$mod>	squareRoot	"TeX"
	eval $func	'r'	<cs$mod>	textrm	"TeX"
	eval $func	'r'	<cso$mod>	mathrm	"TeX"
	
	eval $func	's'	<oz$mod>	varsigma	"TeX"
	eval $func	's'	<o$mod>	sigma	"TeX"
	eval $func	's'	<os$mod>	Sigma	"TeX"
	eval $func	's'	<co$mod>	sum	"TeX"
	eval $func	's'	<cs$mod>	textsl	"TeX"
	
	eval $func	't'	<oz$mod>	triangle	"TeX"
	eval $func	't'	<o$mod>	tau	"TeX"
	eval $func	't'	<os$mod>	dag	"TeX"
	eval $func	't'	<co$mod>	tilde	"TeX"
	eval $func	't'	<cs$mod>	texttt	"TeX"
	eval $func	't'	<cso$mod>	widetilde	"TeX"
	
	eval $func	'u'	<oz$mod>	cup	"TeX"
	eval $func	'u'	<o$mod>	upsilon	"TeX"
	eval $func	'u'	<os$mod>	Upsilon	"TeX"
	eval $func	'u'	<co$mod>	underline	"TeX"
	eval $func	'u'	<cso$mod>	underbrace	"TeX"
	
	eval $func	'v'	<oz$mod>	vee	"TeX"
	eval $func	'v'	<o$mod>	nabla	"TeX"
	eval $func	'v'	<co$mod>	vec	"TeX"
	
	eval $func	'w'	<oz$mod>	wedge	"TeX"
	eval $func	'w'	<o$mod>	omega	"TeX"
	eval $func	'w'	<os$mod>	Omega	"TeX"
	eval $func	'w'	<cs$mod>	textsf	"TeX"
	eval $func	'w'	<cso$mod>	mathsf	"TeX"
	
	eval $func	'x'	<oz$mod>	times	"TeX"
	eval $func	'x'	<o$mod>	xi	"TeX"
	eval $func	'x'	<os$mod>	Xi	"TeX"
	eval $func	'x'	<cs$mod>	ref	"TeX"
	
	eval $func	'y'	<o$mod>	psi	"TeX"
	eval $func	'y'	<os$mod>	Psi	"TeX"
	
	eval $func	'z'	<o$mod>	zeta	"TeX"
	
	eval $func	{'\ '}	<oz$mod>	medium	"TeX"
	eval $func	{'\ '}	<o$mod>	negThin	"TeX"
	eval $func	{'\ '}	<os$mod>	thin	"TeX"
	eval $func	{'\ '}	<co$mod>	thick	"TeX"
	eval $func	{'\ '}	<cs$mod>	quad	"TeX"
	eval $func	{'\ '}	<cso$mod>	qquad	"TeX"
	
	eval $func	','	<oz$mod>	subseteq	"TeX"
	eval $func	','	<o$mod>	leq	"TeX"
	eval $func	','	<os$mod>	subset	"TeX"
	eval $func	','	<co$mod>	subscript	"TeX"
	
	eval $func	'.'	<oz$mod>	supseteq	"TeX"
	eval $func	'.'	<o$mod>	geq	"TeX"
	eval $func	'.'	<os$mod>	supset	"TeX"
	eval $func	'.'	<co$mod>	superscript	"TeX"
	
	eval $func	'/'	<o$mod>	div	"TeX"
	eval $func	'/'	<co$mod>	fraction	"TeX"
	
	eval $func	{'\;'}	<oz$mod>	vdots	"TeX"
	eval $func	{'\;'}	<o$mod>	ldots	"TeX"
	eval $func	{'\;'}	<os$mod>	cdots	"TeX"
	eval $func	{'\;'}	<co$mod>	ddots	"TeX"
	
	eval $func	{'\''}	<co$mod>	acute	"TeX"
	eval $func	{'\''}	<cs$mod>	quotes	"TeX"
	eval $func	{'\''}	<cso$mod>	dblQuotes	"TeX"
	
	eval $func	{'\['}	<o$mod>	brackets	"TeX"
	eval $func	{'\['}	<os$mod>	braces	"TeX"
	eval $func	{'\['}	<co$mod>	bigBrackets	"TeX"
	eval $func	{'\['}	<cso$mod>	bigBraces	"TeX"
	
	eval $func	{'\]'}	<oz$mod>	scriptstyle	"TeX"
	eval $func	{'\]'}	<o$mod>	displaystyle	"TeX"
	eval $func	{'\]'}	<os$mod>	textstyle	"TeX"
	eval $func	{'\]'}	<co$mod>	scriptscriptstyle	"TeX"
	eval $func	{'\]'}	<cso$mod>	bigLeftBrace	"TeX"
	
	eval $func	{'\\'}	<oz$mod>	setMinus	"TeX"
	eval $func	{'\\'}	<o$mod>	backslash	"TeX"
	eval $func	{'\\'}	<os$mod>	absoluteValue	"TeX"
	eval $func	{'\\'}	<co$mod>	bigAbsoluteValue	"TeX"
	eval $func	{'\\'}	<cs$mod>	"oneParameter"	"TeX"
	eval $func	{'\\'}	<cso$mod>	"twoParameters"	"TeX"
	
	eval $func	'`'	<co$mod>	grave	"TeX"
	eval $func	'`'	<cso$mod>	tilde	"TeX"
		
	if {$TeXmodeVars(useDollarSigns)} then {
		eval $func	'4'	<co$mod>	texMath	"TeX"
		eval $func	'4'	<cso$mod>	texDisplaymath	"TeX"
	} else {
		eval $func	'4'	<co$mod>	latexMath	"TeX"
		eval $func	'4'	<cso$mod>	latexDisplaymath	"TeX"
	}
	
	eval $func	'5'	<o$mod>	infty	"TeX"
	
	eval $func	'6'	<o$mod>	sectionMark	"TeX"
	eval $func	'6'	<co$mod>	superscript	"TeX"
	
	eval $func	'7'	<o$mod>	paragraphMark	"TeX"
	
	eval $func	'8'	<oz$mod>	cdot	"TeX"
	eval $func	'8'	<os$mod>	ast	"TeX"
	eval $func	'8'	<co$mod>	bullet	"TeX"
	
	eval $func	'9'	<os$mod>	parentheses	"TeX"
	eval $func	'9'	<co$mod>	bigParentheses	"TeX"
	
	eval $func	'0'	<oz$mod>	emptyset	"TeX"
	
	eval $func	'-'	<oz$mod>	neg	"TeX"
	eval $func	'-'	<o$mod>	sim	"TeX"
	eval $func	'-'	<os$mod>	mp	"TeX"
	eval $func	'-'	<co$mod>	subscript	"TeX"
	
	eval $func	'='	<oz$mod>	approx	"TeX"
	eval $func	'='	<o$mod>	neq	"TeX"
	eval $func	'='	<os$mod>	pm	"TeX"
	eval $func	'='	<co$mod>	bar	"TeX"
	
	eval $func	F5	<oz$mod>	equation	"TeX"
	eval $func	F5	<o$mod>	math	"TeX"
	eval $func	F5	<os$mod>	displaymath	"TeX"
	
	eval $func	F6	<oz$mod>	eqnarray	"TeX"
	eval $func	F6	<o$mod>	myArray	"TeX"
	eval $func	F6	<os$mod>	eqnarrayStar	"TeX"
	
	eval $func	F7	<oz$mod>	description	"TeX"
	eval $func	F7	<o$mod>	enumerate	"TeX"
	eval $func	F7	<os$mod>	itemize	"TeX"
	eval $func	F7	<co$mod>	thebibliography	"TeX"
	
	eval $func	F8	<oz$mod>	note	"TeX"
	eval $func	F8	<o$mod>	slide	"TeX"
	eval $func	F8	<os$mod>	overlay	"TeX"
	
	eval $func	F9	<oz$mod>	tabular	"TeX"
	eval $func	F9	<o$mod>	figure	"TeX"
	eval $func	F9	<os$mod>	table	"TeX"
	
	eval $func	F10	<oz$mod>	quotation	"TeX"
	eval $func	F10	<o$mod>	verbatim	"TeX"
	eval $func	F10	<os$mod>	quote	"TeX"
	eval $func	F10	<co$mod>	verse	"TeX"
	
	eval $func	F11	<oz$mod>	flushright	"TeX"
	eval $func	F11	<o$mod>	flushleft	"TeX"
	eval $func	F11	<os$mod>	center	"TeX"
	
	eval $func	F12	<o$mod>	general	"TeX"
	
}

