#############################################################################
#############################################################################
#
# LaTeX Menu Definition (called from latex.tcl)
#
#############################################################################
#
# Author:  Tom Scavo (trscavo@syr.edu)
#
#############################################################################
#############################################################################

proc interpretMenuItem {menu item} {
	switch $item {
		"Typeset"  {set func "typeset"}
		"LaTeX"  {set func "latex"}
		"BibTeX"  {set func "bibtex"}
		"MakeIndex"  {set func "makeindex"}
		{Next Tab Stop}  {set func "nextTabStop"}
		{Prev Tab Stop}  {set func "prevTabStop"}
		{nth Tab Stop}  {set func {nthTabStop 0}}
		{Next Section}  {set func "nextSection"}
		{Prev Section}  {set func "prevSection"}
		{Next Subsection}  {set func "nextSubsection"}
		{Prev Subsection}  {set func "prevSubsection"}
		"list"  {set func "myList"}
		"array"  {set func "myArray"}
		"eqnarray\*"  {set func "eqnarrayStar"}
		"ellipsis"  {set func "ldots"}
		{TeX logo}  {set func "texLogo"}
		{LaTeX logo}  {set func "latexLogo"}
		{LaTeX2e logo}  {set func "latex2eLogo"}
		"date"  {set func "today"}
		{dag }  {set func "dag"}
		{section mark }  {set func "sectionMark"}
		{paragraph mark }  {set func "paragraphMark"}
		{copyright }  {set func "copyright"}
		{pounds }  {set func "pounds"}
		{double quotes}  {set func "dblQuotes"}
		{marginal note}  {set func "marginalNote"}
		{TeX math}  {set func "texMath"}
		{TeX displaymath}  {set func "texDisplaymath"}
		{LaTeX math}  {set func "latexMath"}
		{LaTeX displaymath}  {set func "latexDisplaymath"}
		{blackboard bold} {set func "blackboardBold"}
		{square root}  {set func "squareRoot"}
		{nth root}  {set func "nthRoot"}
		{one parameter}  {set func "oneParameter"}
		{two parameters}  {set func "twoParameters"}
		{pm }  {set func "pm"}
		{div }  {set func "div"}
		{leq }  {set func "leq"}
		{ll }  {set func "myLl"}
		{geq }  {set func "geq"}
		{gg }  {set func "gg"}
		{approx }  {set func "approx"}
		{neq }  {set func "neq"}
		{join}  {set func "myJoin"}
		{neg }  {set func "neg"}
		{partial }  {set func "partial"}
		{infty }  {set func "infty"}
		{vertical bars}  {set func "absoluteValue"}
		{other delims}  {set func "otherDelims"}
		{half-open interval}  {set func "half-openInterval"}
		{half-closed interval}  {set func "half-closedInterval"}
		{big parentheses}  {set func "bigParentheses"}
		{big brackets}  {set func "bigBrackets"}
		{big braces}  {set func "bigBraces"}
		{big vertical bars}  {set func "bigAbsoluteValue"}
		{other big delims}  {set func "otherBigDelims"}
		{big left brace}  {set func "bigLeftBrace"}
		{other mixed big delims}  {set func "otherMixedBigDelims"}
		{neg thin} {set func "negThin"}
		default {set func $item}
	}
	eval $func
}

proc buildLaTeXMenu {} {
	global latexMenu

	menu -n $latexMenu -m  {
	
		"/TTypeset"
		
		{menu -n Goto -m -p interpretMenuItem {
			"/-LaTeX"
			"BibTeX"
			"MakeIndex"
			"(-"
			{Next Tab Stop}
			{Prev Tab Stop}
			{nth Tab Stop}
			"(-"
			{Next Section}
			{Prev Section}
			{Next Subsection}
			{Prev Subsection}
			}
		}
		
		{menu -n {LaTeX Utilities} -p interpretMenuItem {
			clearTabStops
			convertDollarSigns
			"(-"
			loadSymbolPackage
			}
		}
		
		"(-"
		
		{menu -n Documents -m -p interpretMenuItem {
			"letter"
			"article"
			"report"
			"book"
			"slides"
			"custom"
			"(-"
			"styles"
			"packages"
			"(-"
			"(filecontents"
			}
		}
		
		{menu -n {Page Layout} -m -p interpretMenuItem {
			"maketitle"
			"(-"
			"abstract"
			"titlepage"
			"(-"
			"pagestyle"
			"thispagestyle"
			"pagenumbering"
			"(-"
			"twocolumn"
			"onecolumn"
			}
		}
		
		{menu -n Sectioning -m -p interpretMenuItem {
			"part"
			"chapter"
			"section"
			"subsection"
			"subsubsection"
			"paragraph"
			"subparagraph"
			"(-"
			"appendix"
			}
		}
		
		"(-"
		
		{menu -n {Text Style} -m -p interpretMenuItem {
			"emph"
			"underline"
			"(-"
			"textup"
			"textit"
			"textsl"
			"textsc"
			"(-"
			"textmd"
			"textbf"
			"(-"
			"textrm"
			"textsf"
			"texttt"
			"(-"
			"textnormal"
			}
		}
		
		{menu -n {Text Size} -m -p interpretMenuItem {
			"tiny"
			"scriptsize"
			"footnotesize"
			"small"
			"normalsize"
			"large"
			"Large"
			"LARGE"
			"huge"
			"Huge"
			}
		}
		
		{menu -n International -p interpretMenuItem {
			{}
			{}
			{}
			{}
			{}
			{}
			"(-"
			{}
			{}
			{}
			{}
			{}
			{}
			{}
			{}
			{}
			{}
			{}
			}
		}
		
		{menu -n Environments -m -p interpretMenuItem {
			"enumerate"
			"itemize"
			"description"
			"thebibliography"
			"(-"
			"slide"
			"overlay"
			"note"
			"(-"
			"figure"
			"table"
			"tabular"
			"(-"
			"verbatim"
			"quote"
			"quotation"
			"verse"
			"(-"
			"flushleft"
			"center"
			"flushright"
			"(-"
			"general"
			}
		}
		
		{menu -n Boxes -m -p interpretMenuItem {
			"mbox"
			"makebox"
			"fbox"
			"framebox"
			"(-"
			"newsavebox"
			"sbox"
			"savebox"
			"usebox"
			"(-"
			"raisebox"
			"(-"
			"parbox"
			"minipage"
			"(-"
			"rule"
			}
		}
		
		{menu -n Miscellaneous -m -p interpretMenuItem {
			"ellipsis"
			"en-dash"
			"em-dash"
			{TeX logo}
			{LaTeX logo}
			{LaTeX2e logo}
			"date"
			"(-"
			{dag }
			"ddag"
			{section mark }
			{paragraph mark }
			{copyright }
			{pounds }
			"(-"
			"quotes"
			{double quotes}
			"(-"
			{marginal note}
			"footnote"
			"(-"
			"label"
			"ref"
			"pageref"
			"cite"
			"(-"
			"item"
			"bibitem"
			}
		}
		
		"(-"
		
		{menu -n {Math Modes} -m -p interpretMenuItem {
			{TeX math}
			{TeX displaymath}
			"(-"
			{LaTeX math}
			{LaTeX displaymath}
			}
		}
		
		{menu -n {Math Style} -m -p interpretMenuItem {
			"mathit"
			"mathrm"
			"mathbf"
			"mathsf"
			"mathtt"
			"mathcal"
			"(-"
			"displaystyle"
			"textstyle"
			"scriptstyle"
			"scriptscriptstyle"
			}
		}
		
		{menu -n {Math Environments} -m -p interpretMenuItem {
			"math"
			"displaymath"
			"equation"
			"(-"
			"array"
			"eqnarray*"
			"eqnarray"
			"(-"
			"general"
			}
		}
		
		{menu -n Formulas -m -p interpretMenuItem {
			"subscript"
			"superscript"
			"fraction"
			{square root}
			{nth root}
			"(-"
			{one parameter}
			{two parameters}
			}
		}
	
		{menu -n Greek -m -p interpretMenuItem {
			"alpha"
			"beta"
			"gamma"
			"delta"
			"epsilon"
			"zeta"
			"eta"
			"theta"
			"iota"
			"kappa"
			"lambda"
			"mu"
			"nu"
			"xi"
			"omicron"
			"pi"
			"rho"
			"sigma"
			"tau"
			"upsilon"
			"phi"
			"chi"
			"psi"
			"omega"
			"(-"
			"Gamma"
			"Delta"
			"Theta"
			"Lambda"
			"Xi"
			"Pi"
			"Sigma"
			"Upsilon"
			"Phi"
			"Psi"
			"Omega"
			"(-"
			"varepsilon"
			"vartheta"
			"varpi"
			"varrho"
			"varsigma"
			"varphi"
			}
		}
			
		{menu -n {Binary Operators} -m -p interpretMenuItem {
			{pm }
			"mp"
			"times"
			{div }
			"ast"
			"star"
			"circ"
			"bullet"
			"cdot"
			"cap"
			"cup"
			"uplus"
			"sqcap"
			"sqcup"
			"vee"
			"wedge"
			"setminus"
			"(-"
			"diamond"
			"bigtriangleup"
			"bigtriangledown"
			"triangleleft"
			"triangleright"
			"lhd"
			"rhd"
			"unlhd"
			"unrhd"
			"(-"
			"oplus"
			"ominus"
			"otimes"
			"oslash"
			"odot"
			"(-"
			"bigcirc"
			"dagger"
			"ddagger"
			"amalg"
			"wr"
			}
		}
		
		{menu -n Relations -m -p interpretMenuItem {
			{leq }
			"prec"
			"preceq"
			{ll }
			"subset"
			"subseteq"
			"sqsubset"
			"sqsubseteq"
			"in"
			"vdash"
			"(-"
			{geq }
			"succ"
			"succeq"
			{gg }
			"supset"
			"supseteq"
			"sqsupset"
			"sqsupseteq"
			"ni"
			"dashv"
			"(-"
			"equiv"
			"sim"
			"simeq"
			"asymp"
			{approx }
			"cong"
			{neq }
			"doteq"
			"propto"
			"(-"
			"models"
			"perp"
			"mid"
			"parallel"
			"bowtie"
			"join"
			"smile"
			"frown"
			}
		}
		
		{menu -n Arrows -m -p interpretMenuItem {
			"leftarrow"
			"Leftarrow"
			"rightarrow"
			"Rightarrow"
			"leftrightarrow"
			"Leftrightarrow"
			"mapsto"
			"(-"
			"longleftarrow"
			"Longleftarrow"
			"longrightarrow"
			"Longrightarrow"
			"longleftrightarrow"
			"Longleftrightarrow"
			"longmapsto"
			"(-"
			"hookleftarrow"
			"hookrightarrow"
			"leftharpoonup"
			"rightharpoonup"
			"leftharpoondown"
			"rightharpoondown"
			"rightleftharpoons"
			"leadsto"
			"(-"
			"uparrow"
			"Uparrow"
			"downarrow"
			"Downarrow"
			"updownarrow"
			"Updownarrow"
			"(-"
			"nearrow"
			"searrow"
			"swarrow"
			"nwarrow"
			}
		}
		
		{menu -n Dots -m -p interpretMenuItem {
			"bullet"
			"cdot"
			"(-"
			"ldots"
			"cdots"
			"vdots"
			"ddots"
			}
		}
		
		{menu -n Symbols -m -p interpretMenuItem {
			"aleph"
			"hbar"
			"imath"
			"jmath"
			"ell"
			"wp"
			"Re"
			"Im"
			"mho"
			"(-"
			"angle"
			"backslash"
			"bot"
			"emptyset"
			"exists"
			"forall"
			{infty }
			"nabla"
			{neg }
			{partial }
			"prime"
			"surd"
			"top"
			"(-"
			"Box"
			"Diamond"
			"triangle"
			"clubsuit"
			"diamondsuit"
			"heartsuit"
			"spadesuit"
			"(-"
			"flat"
			"natural"
			"sharp"
			}
		}
			
		{menu -n Functions -m -p interpretMenuItem {
			"arccos"
			"arcsin"
			"arctan"
			"arg"
			"cos"
			"cosh"
			"cot"
			"coth"
			"csc"
			"deg"
			"det"
			"dim"
			"exp"
			"gcd"
			"hom"
			"inf"
			"ker"
			"lg"
			"lim"
			"liminf"
			"limsup"
			"ln"
			"log"
			"max"
			"min"
			"Pr"
			"sec"
			"sin"
			"sinh"
			"sup"
			"tan"
			"tanh"
			"(-"
			"bmod"
			"pmod"
			}
		}
	
		{menu -n {Large Operators} -m -p interpretMenuItem {
			"sum"
			"prod"
			"coprod"
			"int"
			"oint"
			"bigcap"
			"bigcup"
			"bigsqcap"
			"bigsqcup"
			"bigvee"
			"bigwedge"
			"bigodot"
			"bigotimes"
			"bigoplus"
			"biguplus"
			}
		}
		
		{menu -n Delimiters -m -p interpretMenuItem {
			"parentheses"
			"brackets"
			"braces"
			{vertical bars}
			{other delims}
			"(-"
			{half-open interval}
			{half-closed interval}
			"(-"
			{big parentheses}
			{big brackets}
			{big braces}
			{big vertical bars}
			{other big delims}
			"(-"
			{big left brace}
			{other mixed big delims}
			}
		}
			
		{menu -n {Math Accents} -m -p interpretMenuItem {
			"acute"
			"bar"
			"breve"
			"check"
			"ddot"
			"dot"
			"grave"
			"hat"
			"tilde"
			"vec"
			"(-"
			"widehat"
			"widetilde"
			"(-"
			"imath"
			"jmath"
			}
		}
		
		{menu -n Grouping -m -p interpretMenuItem {
			"underline"
			"overline"
			"underbrace"
			"overbrace"
			"(-"
			"stackrel"
			}
		}
		
		{menu -n Spacing -m -p interpretMenuItem {
			{neg thin}
			"thin"
			"medium"
			"thick"
			"(-"
			"quad"
			"qquad"
			"(-"
			"hspace"
			"vspace"
			"(-"
			"hfill"
			"vfill"
			"(-"
			"smallskip"
			"medskip"
			"bigskip"
			}
		}
	}
}
